/*
  AntiRight (c) 2002-2004 
  Jeffrey Bedard <jefbed@e-list.net>
 
  This file is part of AntiRight.

  AntiRight is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  AntiRight is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with AntiRight; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "arshell.h"

void 
arshell_search_ok_cb(Widget widget, 
		     XtPointer client_data, 
		     XtPointer call_data)
{
  XmTextPosition position;
  XmSelectionBoxCallbackStruct *prompt_data;
  char *search_string;
  
  assert(call_data != NULL);
  
  prompt_data=call_data;

  XmStringGetLtoR(prompt_data->value,
		  XmSTRING_DEFAULT_CHARSET,
		  &search_string);

  assert(search_string != NULL);

  XmTextFindString((Widget)client_data,
		   XmTextGetInsertionPosition((Widget)client_data),
		   search_string,
		   XmTEXT_FORWARD,
		   &position);

  XmTextSetInsertionPosition((Widget)client_data, 
			     position);

  XtFree(search_string);
  widget=NULL;
  call_data=NULL;
}

void 
arshell_search_cb(Widget widget, 
		     XtPointer client_data, 
		     XtPointer call_data)
{
  Widget prompt;

  ARCLARG;
  antiright_set(XmNtitle, (XtArgVal)"Search");
  
  prompt=
    XmCreatePromptDialog(antiright_parent_widget,
			 "Search",
			 antiright_arguments,
			 antiright_number_arguments);
  
  XtAddCallback(prompt,
		XmNokCallback,
		arshell_search_ok_cb,
		(Widget)client_data);
  XtAddCallback(prompt,
		XmNhelpCallback,
		antiright_help_cb,
		"Enter the text that you want to find in the text field.");
  XtManageChild(prompt);

  widget=(Widget)NULL;
  call_data=(XtPointer)NULL;
}

/* This is a simple method of printing the current file.  */
void 
arshell_print_cb()
{
  char *command;
  char *print_string;

  print_string=
    antiright_pipe_read("echo -n `ACE -r print`");
  asprintf(&command, "%s %s &", print_string, arshell_filename);
  free(print_string);
  system(command); /* Print the file.  */
  free(command); /* Free the unneeded memory.  */
}

void
arshell_value_changed_cb()
{
  char *title_string;

  asprintf(&title_string,
	   "%s [%s]",
	   arshell_filename,
	   "modified");
  XtVaSetValues(antiright_parent_widget,
		XmNtitle, title_string,
		NULL);
  free(title_string);

  arshell_modified_flag=True;
}

void
arshell_text_return_cb(Widget parent_widget,
		       XtPointer client_data,
		       XtPointer call_data)
{
  char* text_string=XmTextGetString(parent_widget);
  printf("%s %s", (char*)client_data, text_string);
  free(text_string);
  fflush(stdout);
  call_data=(XtPointer)NULL;
  exit(0);
}

void
arshell_save_work_area()
{
  char* text_string;
  FILE* file_pointer=fopen(arshell_filename, "w");
  assert(arshell_text_widget != NULL);
  text_string=XmTextGetString(arshell_text_widget);
  assert(file_pointer != NULL);
  fprintf(file_pointer, "%s\n", text_string);
  XtFree(text_string);
  fclose(file_pointer);
  if(arshell_modified_flag)
    arshell_modified_flag=False;
  antiright_set_title(arshell_filename);
}

void
arshell_set_work_area_arguments()
{
  ARCLARG;
  antiright_set(XmNscrollHorizontal, False);
  antiright_set(XmNscrollVertical, True);
  antiright_set(XmNeditMode, XmMULTI_LINE_EDIT);
  antiright_set(XmNwordWrap, True);
  antiright_set(XmNcolumns, 80);
  antiright_set(XmNrows, 24);
}

void
arshell_setup_work_menubar(struct antiright_menubar_structure* menubar)
{
  ARCLARG;
  if(menubar->file_widget != NULL)
    {
      antiright_button(menubar->file_widget, "Save",
		       arshell_save_work_area, NULL);
      menubar->edit_widget=antiright_edit_menu(menubar->menubar_widget, 
					       arshell_text_widget);
      antiright_separator(menubar->edit_widget);
      antiright_button(menubar->edit_widget, "Search",
		       arshell_search_cb, arshell_text_widget);
    }
}


void
arshell_create_text_work_area(Widget parent_widget,
			      struct antiright_menubar_structure* menubar)
{
  arshell_set_work_area_arguments();
  arshell_text_widget=
    XmCreateScrolledText(parent_widget, "arshell_text_widget",
			 antiright_arguments, antiright_number_arguments);
  XtManageChild(arshell_text_widget);
  antiright_defined_popup_menu(arshell_text_widget);
  XtAddCallback(arshell_text_widget, XmNvalueChangedCallback, 
		arshell_value_changed_cb, NULL);
  arshell_setup_work_menubar(menubar);
}

void
arshell_create_default_text_label(Widget form_widget, 
				  char* label_string)
{
  ARCLARG;
  antiright_form_positions(0, 100, 0, 25);
  antiright_label(form_widget, label_string);
}

Widget
arshell_create_default_text_field(Widget mainwindow_widget,
				  char* return_string)
{
  Widget text_widget;
  ARCLARG;
  antiright_form_positions(0, 100, 25, 100);
  antiright_set(XmNeditMode, XmSINGLE_LINE_EDIT);
  antiright_set(XmNscrollHorizontal, False);
  text_widget=XmCreateText(mainwindow_widget, "text_widget", 
			   antiright_arguments, antiright_number_arguments);
  XtManageChild(text_widget);
  XtAddCallback(text_widget, XmNactivateCallback, 
		arshell_text_return_cb, return_string);
  return(text_widget);
}

void
arshell_set_work_area_from_stdin()
{
  char buffer[BUFSIZ];
  assert(arshell_text_widget != NULL);
  while(fgets(buffer, BUFSIZ, stdin) != NULL)
    XmTextInsert(arshell_text_widget, 
		 XmTextGetInsertionPosition(arshell_text_widget), buffer);
}


void
arshell_create_default_text_entry(Widget widget_row_widget,
				  int* counter_int,
				  char** argv)
{
  Widget form_widget;
  
  ARCLARG;
  form_widget=antiright_form(widget_row_widget);
  arshell_create_default_text_label(form_widget, argv[(*counter_int)]);
  arshell_create_default_text_field(form_widget, argv[(*counter_int)]);
}
